﻿using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using Infragistics.UltraChart.Resources.Appearance;
using Infragistics.UltraChart.Shared.Styles;
using Infragistics.UltraChart.Core;
using Infragistics.UltraChart.Core.Primitives;

namespace NullHandlingScatterComposite
{
    public partial class Form1 : Form
    {
        #region Members

        PaintElement paintElement = null;
        LineStyle lineStyle = null;

        #endregion // Members

        #region Constructor

        public Form1()
        {
            InitializeComponent();
            this.InitChart();
            this.ultraChart1.FillSceneGraph += new Infragistics.UltraChart.Shared.Events.FillSceneGraphEventHandler(ultraChart1_FillSceneGraph);
        }

        #endregion // Constructor

        #region Properties

        #region PE
        private PaintElement PE
        {
            get
            {
                if (this.paintElement == null)
                {
                    this.paintElement = new PaintElement(Color.LightBlue);
                    paintElement.Hatch = Infragistics.UltraChart.Shared.Styles.FillHatchStyle.Horizontal;
                    paintElement.StrokeWidth = 2;
                }
                return this.paintElement;
            }
        }
        #endregion // PE

        #region LineStyle
        private LineStyle LineStyle
        {
            get
            {
                if (this.lineStyle == null)
                {

                    this.lineStyle = new LineStyle();
                    this.lineStyle.DrawStyle = LineDrawStyle.Solid;
                }
                return this.lineStyle;
            }
        }
        #endregion // LineStyle

        #endregion // Properties

        #region Methods

        #region InitChart

        private void InitChart()
        {
            this.ultraChart1.ChartType = ChartType.Composite;
            
            ChartArea area = new ChartArea();
            this.ultraChart1.CompositeChart.ChartAreas.Add(area);

            AxisItem aAxisX = new AxisItem();
            aAxisX.OrientationType = AxisNumber.X_Axis;
            aAxisX.DataType = AxisDataType.Numeric;
            aAxisX.Labels.ItemFormatString = "<DATA_VALUE:0.#>";
            aAxisX.Labels.Orientation = TextOrientation.VerticalLeftFacing;
            aAxisX.Extent = 40;

            AxisItem aAxisY = new AxisItem();
            aAxisY.OrientationType = AxisNumber.Y_Axis;
            aAxisY.DataType = AxisDataType.Numeric;
            aAxisY.Labels.ItemFormatString = "<DATA_VALUE:0.#>";

            area.Axes.Add(aAxisX);
            area.Axes.Add(aAxisY);

            XYSeries xySeries = this.GetXYSeriesBound();
            this.ultraChart1.CompositeChart.Series.Add(xySeries);

            ChartLayerAppearance lineLayer = new ChartLayerAppearance();
            lineLayer.ChartType = ChartType.ScatterChart;
            ((ScatterChartAppearance)lineLayer.ChartTypeAppearance).ConnectWithLines = false; // draw our own lines
            ((ScatterChartAppearance)(lineLayer.ChartTypeAppearance)).NullHandling = NullHandling.Zero; // make the chart think it is drawing segments to 0 for nulls
            ((ScatterChartAppearance)(lineLayer.ChartTypeAppearance)).Icon = SymbolIcon.Circle;
            ((ScatterChartAppearance)(lineLayer.ChartTypeAppearance)).IconSize = SymbolIconSize.Small;
            
            xySeries.PEs.Add(this.PE);
            
            lineLayer.ChartArea = area;
            lineLayer.AxisX = aAxisX;
            lineLayer.AxisY = aAxisY;

            lineLayer.Series.Add(xySeries);
            this.ultraChart1.CompositeChart.ChartLayers.Add(lineLayer);
        }

        #endregion // InitChart

        #region GetData
        
        private static DataTable GetData()
        {
            DataTable table = new DataTable();
            table.Columns.Add("Label Column", typeof(string));
            table.Columns.Add("Value Column", typeof(double));
            table.Columns.Add("Another Value Column", typeof(double));
            table.Rows.Add(new object[] { "Point A", 1.0, 3.0 });
            table.Rows.Add(new object[] { "Point B", 2.0, 2.0 });
            table.Rows.Add(new object[] { "Point C", null, 1.0 });
            table.Rows.Add(new object[] { "Point D", 4.0, 2.0 });
            table.Rows.Add(new object[] { "Point E", 5.0, 3.0 });

            return table;
        }
        
        #endregion // GetData

        #region GetXYSeriesBound

        private XYSeries GetXYSeriesBound()
        {
            XYSeries series = new XYSeries();
            series.Label = "Series C";
            // this code populates the series from an external data source
            DataTable table = GetData();
            series.Data.DataSource = table;
            series.Data.LabelColumn = "Label Column";
            series.Data.ValueXColumn = "Value Column";
            series.Data.ValueYColumn = "Another Value Column";
            return series;
        }

        #endregion // GetXYSeriesBound

        #endregion // Methods
        
        #region Event Handlers

        #region ultraChart1_FillSceneGraph

        void ultraChart1_FillSceneGraph(object sender, Infragistics.UltraChart.Shared.Events.FillSceneGraphEventArgs e)
        {
            SceneGraph scene = e.SceneGraph;
            
            // find the PointSets to draw lines for each segment.
            List<PointSet> pointsets = new List<PointSet>(5);
            foreach (Primitive p in scene)
            {
                if (p is PointSet)
                {
                    pointsets.Add(p as PointSet);
                }
            }

            // for each PointSet draw a PolyLine.
            if (pointsets.Count > 0)
            {
                Polyline pLine = null;
                foreach (PointSet pointSet in pointsets)
                {
                    pLine = new Polyline(pointSet.points, this.LineStyle);
                    pLine.Caps = PCaps.Skin | PCaps.HitTest | PCaps.Tooltip;
                    pLine.PE = this.PE;
                    pLine.splineTension = 0f;
                    scene.Add(pLine);
                }
            }
        }

        #endregion // ultraChart1_FillSceneGraph

        #endregion // Event Handlers
    }
}
